#!/bin/bash 
#set -x
################################################# 
# USB sqlite3 database update script
# TODO:
#	# This script is invoked by udev, you should add these udev rules in /etc/udev/rules.d/96-moxa.rules
#	KERNEL=="2-1.1", ACTION=="add",    RUN+="/etc/mxAgent/scripts/snmp/mx_usb_update.sh"
#	KERNEL=="2-1.1", ACTION=="remove", RUN+="/etc/mxAgent/scripts/snmp/mx_usb_update.sh"
#	KERNEL=="2-1.2", ACTION=="add",    RUN+="/etc/mxAgent/scripts/snmp/mx_usb_update.sh"
#	KERNEL=="2-1.2", ACTION=="remove", RUN+="/etc/mxAgent/scripts/snmp/mx_usb_update.sh"
#	
#################################################

export MX_DB_PATH=/dev/shm/snmp.db
export MX_SCRIPT_PATH=/etc/mxAgent/scripts/

# Please use these predefined variable
. $MX_SCRIPT_PATH/mx_global.sh
. $MX_SCRIPT_PATH/snmp/global.sh

table=$mx_usb_table
control_table=$mx_usb_control_table
gb_debug_enable=0
gb_print_enable=0

# parse those patterns
#	1. Var=val
#	1. Var= val #include a space
str_get()
{
	local str=$1
	local var=$2
	echo $str | sed -n "s:.*$var=\s\?\([a-z0-9A-Z.]*\).*:\1:p"
}

# like str_get, but get whole token to the tail
str_get_last()
{
	local str=$1
	local var=$2
	echo $str | sed -n "s:.*$var=\s\?\(.*\).*:\1:p"
}

usb_handle_trap()
{
	local enable_trap=`gb_sql_query "SELECT enableTrap FROM $control_table"`
	local trap_id=".1.3.6.1.4.1.8691.17.1.6.4.2.1"

	if [ "$enable_trap" -eq "0" ]; then
		return 
	fi 

	gb_sql_import_row_var "SELECT trapIp, trapCommunity FROM $mx_sys_table" 
	gb_print "send trap ip=$trapIp"
	# put in the background?
	snmptrap -v 2c -c $trapCommunity $trapIp "" $trap_id 
}

#################################################
# STARTING POINT:
#################################################

i=0
flag_first_line=0
# use to skip level 00 device
flag_skip_level0=0
flag_device_change=0


gb_print "$0: Scan------------------------------------"

while read line; do

	# skip the first line
	if [ -z "$line" -a $flag_first_line -eq 0 ]; then
		flag_first_line=1
		continue
	fi

	# skip lines until the end of a device (get the empty line)
	if [ $flag_skip_level0 -eq 1 -a ! -z "$line" ]; then
		continue
	fi

	type=`echo $line | sed -n "s:\(\w*\).*:\1:p"`
	case "$type" in
		T) 
			bus=`str_get "$line" "Bus"`
			level=`str_get "$line" "Lev"`
			port=`str_get "$line" "Port"`
			if [ "$level" == "00" ]; then
				flag_skip_level0=1
			fi
			;;
		D) 
			device_cls=`str_get "$line" "Cls"`
			;;
		P) 
			vendor_id="0x`str_get "$line" "Vendor"`"
			product_id="0x`str_get "$line" "ProdID"`"
			;;
		S)      
			sub_type="`echo $line | sed -n "s:S\:\s*\(.\w*\)=.*:\1:p"`"
			if [ "$sub_type" == "Manufacturer" ]; then
				manufacturer=`str_get_last "$line" "Manufacturer"`
			elif [ "$sub_type" == "Product" ]; then
				product=`str_get_last "$line" "Product"`
			fi
			;;
		I)	
			if [ -z "$interface_cls" ]; then
				interface_cls=`str_get "$line" "Cls"`
			else
				interface_cls=$interface_cls:`str_get "$line" "Cls"`
			fi
			;;
		"") 
			# handle a empty line (end of a entry)

			if [ $flag_skip_level0 -eq 1 ]; then
				flag_skip_level0=0
			fi

			if [ $level == "02" ]; then
				# not show devices at level 02
				gb_debug "-------------------------------"
				gb_debug "bus=$bus"
				gb_debug "level=$level"
				gb_debug "port=$port"
				gb_debug "device_cls=$device_cls"
				gb_debug "vendor_id=$vendor_id"
				gb_debug "product_id=$product_id"
				gb_debug "manufacturer=$manufacturer"
				gb_debug "product=$product"
				gb_debug "interface_cls=$interface_cls"
				gb_debug "-------------------------------"

				# 
				# Write to db
				#
				vendor="$vendor_id ($manufacturer)"
				product="$product_id ($product)"


				active_class=1	#other
				if [ "$device_cls"=="00" -a "$interface_cls" == "03" ]; then
					active_class=2 #hid
				elif [ "$device_cls"=="00" -a "$interface_cls" == "08" ]; then
					active_class=3 #mass
				fi
				
				#check if exsited 
				gb_print "vendor=$vendor product=$product active_class=$active_class bus=$bus port=$port"
				id=`gb_sql_query "SELECT id FROM $table WHERE vendor='$vendor' AND product='$product' AND bus='$bus' AND port='$port'"`
				if [ -z "$id" ]; then 
					# new entry
					gb_print  "insert..."
					gb_sql_query "INSERT into $table VALUES (null, '$vendor', '$product', $active_class, $bus, $port, 1)"
					flag_device_change=1
				else 
					# update exited column
					gb_print "set existed = 1"
					gb_sql_query "UPDATE $table set isExisted=1 WHERE id=$id"
				fi


				# must clear data
				unset level device_cls vendor_id product_id manufacturer interface_cls
			fi
			;;
	esac 
	i=`expr $i + 1`

done < /proc/bus/usb/devices

# delete unexisted entries
ids=`gb_sql_query "SELECT * FROM $table WHERE isExisted=0"`
if [ ! -z "$ids" ]; then
	#gb_print "ids=$ids need to delete, table change"
	flag_device_change=1
	gb_sql_query "DELETE FROM $table WHERE isExisted=0"
fi

# clear isExisted columns
gb_sql_query "UPDATE $table SET isExisted=0"

# send trap 
if [ $flag_device_change -eq 1 ]; then
	usb_handle_trap
fi
